;---------------------------------------------------------------;
;                                                               ;
; Banchor                                                       ;
; Object Routines                                               ;
;                                                               ;
;---------------------------------------------------------------;

;------------------------------------------------
; checkObjectOnScreen - Check to see if an object is on the screen
;
; Input:    HL => X, Y Coords
;           A = -(Width-1)
;           B = -(Height-1)
; Output:   CA = 1 if object isn't on the screen
;------------------------------------------------
checkObjectOnScreen:
        ld      (_@objectWidth),a
        ld      a,b
        ld      (_@objectHeight),a
        ld      a,(hl)
        cp      127
        jr      c,objectXOK
        cp      $00
_@objectWidth                           = $-1
        jr      c,objectNotOnScreen
objectXOK:
        inc     hl
        ld      a,(hl)
        cp      63
        jr      c,objectOnScreen
        cp      $00
_@objectHeight                          = $-1
        jr      c,objectNotOnScreen
objectOnScreen:
        or      a
        ret
objectNotOnScreen:
        scf
        ret

;------------------------------------------------
; checkCollision - Check for collisions between two objects
;
; Author:   Patrick Davidson (http://pad.calc.org)
;           Modified slightly by James Vernon
; Input:    (collide1) = Object 1 [x, width, y, height]
;           (collide2) = Object 2 [x, width, y, height]
; Output:   CA = 1 if they are colliding
;------------------------------------------------
checkCollision:
        ld      hl,collide1
        ld      de,(collide2)
        ld      a,(hl)
        sub     e
        jr      c,checkCollision1
        cp      d
        ret     nc
        inc     hl
        jr      checkCollision2
checkCollision1:
        neg
        inc     hl
        cp      (hl)
        ret     nc
checkCollision2:
        ld      de,(collide2+2)
        inc     hl
        ld      a,(hl)
        sub     e
        jr      c,checkCollision3
        cp      d
        ret
checkCollision3:
        neg
        inc     hl
        cp      (hl)
        ret

;------------------------------------------------
; checkPlayerEnemyCollisions - Check for collisions between the player and the enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
checkPlayerEnemyCollisions:
        call    loadPlayerToCollide1
        ld      b,MAX_ENEMIES
        ld      ix,enemyTable
checkPECollisions:
        push    bc
        ld      a,(ix+E_DIR)
        or      a
        jr      z,endCheckPECollisions
        call    loadEnemyToCollide2
        call    checkCollision
        jr      nc,endCheckPECollisions
        ld      l,(ix+E_DIR)
        bit     7,l
        jr      nz,stopEnemyBeingCreated
#ifndef INVINCIBLE
        ld      a,(hurt)
        or      a
        jr      nz,endCheckPECollisions
        ld      hl,(enemyType)
        dec     l
        ld      h,0
        ld      de,enemyDamage
        add     hl,de
        ld      b,(hl)
        call    calculateDamage
        call    decPlayerHearts
        ld      a,INI_HURT
        ld      (hurt),a
#endif
endCheckPECollisions:
        pop     bc
        ld      de,ENEMY_ENTRY_SIZE
        add     ix,de
        djnz    checkPECollisions
        ret
stopEnemyBeingCreated:
        ld      (ix+E_DIR),0
        jr      endCheckPECollisions

;------------------------------------------------
; checkPlayerBulletCollisions - Check for collisions between Cain and any bullets
;
; Input:    None
; Output:   None
;------------------------------------------------
checkPlayerBulletCollisions:
        call    loadPlayerToCollide1
        ld      b,3
        ld      ix,bulletTable
checkPBCollisions:
        push    bc
        ld      a,(ix+B_DIR)
        or      a
        jr      z,endCheckPBCollisions
        bit     7,a
        jr      nz,endCheckPBCollisions
        call    loadBulletToCollide2
        call    checkCollision
        jr      nc,endCheckPBCollisions
        ld      a,(hurt)
        or      a
        jr      nz,endCheckPBCollisions
        ld      a,(ironShield)
        or      a
        jr      nz,blockBullet
        ld      a,(woodenShield)
        or      a
        jr      z,noShield
        ld      a,(enemyType)
        ld      hl,woodenBlock
        ld      bc,NUM_WOODEN
        cpir
        jr      z,blockBullet
noShield:
#ifndef INVINCIBLE
        ld      b,BULLET_DAMAGE
        call    calculateDamage
        call    decPlayerHearts
        ld      a,INI_HURT
        ld      (hurt),a
#endif
endCheckPBCollisions:
        pop     bc
        ld      de,BULLET_ENTRY_SIZE
        add     ix,de
        djnz    checkPBCollisions
        ret
blockBullet:
        ld      (ix+B_DIR),0
        jr      endCheckPBCollisions

;------------------------------------------------
; checkSwordEnemyCollisions - Check for collisions between player's sword and his enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
checkSwordEnemyCollisions:
        call    loadSwordToCollide1
        ld      b,MAX_ENEMIES
        ld      ix,enemyTable
checkSECollisions:
        push    bc
        ld      a,(ix+E_DIR)
        or      a
        jr      z,endCheckSECollisions
        ld      a,(attacking)
        or      a
        jr      z,notHurtingEnemy
        inc     a
        jr      z,notHurtingEnemy
        call    loadEnemyToCollide2
        call    checkCollision
        jr      nc,notHurtingEnemy
        bit     7,(ix+E_DIR)
        jr      nz,killEnemy
        ld      (ix+E_HURT),1
        ld      hl,attackStrength
        ld      a,(ix+E_HEALTH)
        sub     (hl)
        ld      (ix+E_HEALTH),a
        jr      nc,endCheckSECollisions
        ld      hl,(enemyType)
        dec     l
        ld      h,0
        ld      de,enemyGold
        add     hl,de
        ld      l,(hl)
        ld      h,0
        call    _mulHL10
        call    addGold
        ld      b,(ix+E_X)
        ld      c,(ix+E_Y)
        call    newAnim
killEnemy:
        ld      (ix+E_DIR),0
        jr      endCheckSECollisions
notHurtingEnemy:
        ld      (ix+E_HURT),0
endCheckSECollisions:
        pop     bc
        ld      de,ENEMY_ENTRY_SIZE
        add     ix,de
        djnz    checkSECollisions
        ret

.end
